<?php
/**
 * Mahara: Electronic portfolio, weblog, resume builder and social networking
 * Copyright (C) 2006-2012 Catalyst IT Ltd and others; see:
 *                         http://wiki.mahara.org/Contributors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    mahara
 * @subpackage blocktype-scratch
 * @author     Gregor Anzelj
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
 * @copyright  (C) 2012 Gregor Anzelj, gregor.anzelj@gmail.com
 */

defined('INTERNAL') || die();

class PluginBlocktypeScratch extends PluginBlocktype {

    public static function get_title() {
        return get_string('title', 'blocktype.file/scratch');
    }

    public static function get_description() {
        return get_string('description', 'blocktype.file/scratch');
    }

    public static function get_categories() {
        return array('fileimagevideo');
    }

    public static function render_instance(BlockInstance $instance, $editing=false) {
        $configdata = $instance->get('configdata'); // this will make sure to unserialize it for us
              
        if (empty($configdata['artefactid'])) {
            return '';
        }
		
		$docfile  = get_config('docroot') . 'artefact/file/blocktype/scratch/projects/' . $configdata['artefactid'];
		$datafile = get_config('dataroot') . 'artefact/file/originals/' . $configdata['artefactid'] . '/' . $configdata['artefactid'];
		if (!file_exists($docfile)) {
			if (file_exists($datafile)) {
				// Copy Scratch project file from $DATAROOT... to $DOCROOT...
				// It must be accessible through relative path or else the ScratchApplet.jar will not load it!
				$fr = fopen($datafile, 'rb');
				$filesize = filesize($datafile);
				$contents = fread($fr, $filesize);
				fclose($fr);
				$fw = fopen($docfile, 'wb');
				fwrite($fw, $contents);
				fclose($fw);			
			} else {
				// This should never happen
				throw new ArtefactNotFoundException(get_string('scratchprojectdoesnotexist', 'blocktype.file/scratch'));
			}
		}
		
		$artefact = $instance->get_artefact_instance($configdata['artefactid']);
		$url      = get_config('wwwroot') . 'artefact/file/download.php?file='. $artefact->get('id') . '&view=' . $instance->get('view').'&e=.sb';
		$base     = get_config('wwwroot') . 'artefact/file/download.php?file='. $artefact->get('id') . '&view=' . $instance->get('view').'/';
		$width    = (!empty($configdata['width'])) ? hsc($configdata['width']) : '482';
		$height   = (!empty($configdata['height'])) ? hsc($configdata['height']) : '387';

		$result  = '';
		$result .= '<div class="scratch-container center">';
		$result .= '<applet id="ProjectApplet" style="display:block" code="ScratchApplet" codebase="' . get_config('wwwroot') . 'artefact/file/blocktype/scratch/" archive="ScratchApplet.jar" height="' . $height . '" width="' . $width . '">';
		$result .= '<param name="project" value="projects/'. $configdata['artefactid'] .'">';
		$result .= '</applet>';
		$result .= '<a href="' . $url . '">' . hsc($artefact->get('title')) . '</a>';
		$result .= '</div>';	        
        return $result;
    }
    
    public static function delete_instance(BlockInstance $instance) {
        $configdata = $instance->get('configdata'); // this will make sure to unserialize it for us
		
		if (isset($configdata['artefactid']) && !empty($configdata['artefactid'])) {
			// Delete Scratch project file from $DOCROOT, because we're deleting the block that used it...
			$docfile = get_config('docroot') . 'artefact/file/blocktype/scratch/projects/' . $configdata['artefactid'];
			if (file_exists($docfile)) {
				unlink($docfile);
			} else {
				// This should never happen
				throw new ArtefactNotFoundException(get_string('scratchprojectdoesnotexist', 'blocktype.file/scratch'));
			}
		}
	}
	
    public static function has_instance_config() {
        return true;
    }

    public static function instance_config_form($instance) {
        $configdata = $instance->get('configdata');
        safe_require('artefact', 'file');
        $instance->set('artefactplugin', 'file');
        return array(
            'artefactid' => self::filebrowser_element($instance, (isset($configdata['artefactid'])) ? array($configdata['artefactid']) : null),
			'oldartefactid' => array(
				'type' => 'hidden',
				'value' => (isset($configdata['artefactid'])) ? $configdata['artefactid'] : null,
			),
            'width' => array(
                'type' => 'text',
                'title' => get_string('width', 'blocktype.file/scratch'),
                'size' => 4,
                'description' => get_string('widthdescription', 'blocktype.file/scratch'),
                'defaultvalue' => (isset($configdata['width'])) ? $configdata['width'] : '482',
            ),
           'height' => array(
                'type' => 'text',
                'title' => get_string('height', 'blocktype.file/scratch'),
                'size' => 4,
                'description' => get_string('heightdescription', 'blocktype.file/scratch'),
                'defaultvalue' => (isset($configdata['height'])) ? $configdata['height'] : '387',
            ),
        );
    }

    public static function instance_config_validate(Pieform $form, $values) {
		if (!empty($values['oldartefactid']) && $values['oldartefactid'] != $values['artefactid']) {
			// Delete Scratch project file from $DOCROOT, because user selected other Scratch project for embedding...
			$docfile = get_config('docroot') . 'artefact/file/blocktype/scratch/projects/' . $values['oldartefactid'];
			if (file_exists($docfile)) {
				unlink($docfile);
			} else {
				// This should never happen
				throw new ArtefactNotFoundException(get_string('scratchprojectdoesnotexist', 'blocktype.file/scratch'));
			}
		}
		return $values;
	}
	
    public static function filebrowser_element(&$instance, $default=array()) {
        $element = ArtefactTypeFileBase::blockconfig_filebrowser_element($instance, $default);
        $element['title'] = get_string('file', 'artefact.file');
        $element['name'] = 'artefactid';
        $element['config']['selectone'] = true;
        $element['filters'] = array(
            'artefacttype'    => array('file'),
            //'filetype'        => self::get_allowed_mimetypes(),
			'filetype' => 'sb',
        );
        return $element;
    }

    public static function artefactchooser_element($default=null) {
        return array(
            'name'  => 'artefactid',
            'type'  => 'artefactchooser',
            'title' => get_string('file', 'artefact.file'),
            'defaultvalue' => $default,
            'blocktype' => 'scratch',
            'limit' => 10,
            'artefacttypes' => array('file'),
            'template' => 'artefact:file:artefactchooser-element.tpl',
        );
    }

    public static function default_copy_type() {
        return 'full';
    }
    
 }

?>